﻿using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using PF.Foundation.VsxFactory;
using System.IO;
using System.Drawing.Imaging;

namespace PF.ImageViewerDemo
{
    /// <summary>
    /// Simple class to show the using of the VSX Software Interface. The program shows how to receive and visualize images and how to save them on disk.
    /// </summary>
    /// <remarks>
    /// @author     Holger Jung
    /// @date       15.11.2012
    /// @copyright	2012 Pepperl + Fuchs GmbH, Mannheim<br/>
    ///             Proprietary material, all rights reserved, use or disclosure forbidden.
    /// </remarks>
    public partial class MainForm : Form
    {
        PFVsxFactoryVCCustom _sensor;
        bool _saveImages;
        int _offsetCounter;

        /// <summary>
        /// Constructor of the GUI.
        /// </summary>
        /// <remarks>
        /// Creates a new instance of the PFVsxFactoryVCCustom class and adds an image event handler to it.
        /// </remarks>
        public MainForm()
        {
            InitializeComponent();
            _sensor = new PFVsxFactoryVCCustom();
            // Add event. See event function for detailed descriptions.
            _sensor.ImageReceived += new ImageReceivedEventHandler(_sensor_ImageReceived);
            _saveImages = false;

            // Try to load parameters from settings file -> you do not have to refill parameters all the time when starting GUI.
            try
            {
                textBox_Ip.Text = PF.ImageViewerDemo.Properties.Settings.Default.Ip;
                textBox_SaveFolder.Text = PF.ImageViewerDemo.Properties.Settings.Default.SaveFolder;
                textBox_filenamePrefix.Text = PF.ImageViewerDemo.Properties.Settings.Default.FilePrefix;
                textBox_ExampleFileName.Text = textBox_filenamePrefix.Text + string.Format(numberFormat, numericUpDown_ImageStartNumber.Value) + ".bmp";
            }
            catch { }
        }

        /// <summary>
        /// Opens a TCP/IP connection with the sensor.
        /// </summary>
        /// <param name="sender">The event sender object (here: Connect button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_Connect_Click(object sender, EventArgs e)
        {
            // Call the connect-function with a valid ip and port to start connection.
            if (_sensor.Connect(textBox_Ip.Text, 50005))
                MessageBox.Show("Connecting successful.");
            else
            {
                MessageBox.Show("Error connecting with sensor.");
                return;
            }
            // If connecting was successful, call for all parameters from sensor to get them in the local database. The ParameterDataReceived event is called
            // when parameters are received.
            _sensor.GetAllParametersFromSensor();

            button_connect.Enabled = false;
            button_disconnect.Enabled = true;
            button_TriggerSensor.Enabled = true;
        }

        /// <summary>
        /// Disconnects TCP/IP connection.
        /// </summary>
        /// <param name="sender">The event sender object (here: Disconnect button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_Disconnect_Click(object sender, EventArgs e)
        {
            if (_sensor.Connected)
                _sensor.Disconnect();

            button_connect.Enabled = true;
            button_disconnect.Enabled = false;
            button_TriggerSensor.Enabled = false;

            // Set grabbing flag to false
            button_StopGrabbing_Click(sender, e);
        }

        /// <summary>
        /// When closing GUI, disconnect sensor and try saving parameters to settings file.
        /// </summary>
        /// <param name="e">The cancel event.</param>
        protected override void OnClosing(CancelEventArgs e)
        {
            if (_sensor.Connected)
                _sensor.Disconnect();

            try
            {
                PF.ImageViewerDemo.Properties.Settings.Default.Ip = textBox_Ip.Text;
                PF.ImageViewerDemo.Properties.Settings.Default.SaveFolder = textBox_SaveFolder.Text;
                PF.ImageViewerDemo.Properties.Settings.Default.FilePrefix = textBox_filenamePrefix.Text;
                PF.ImageViewerDemo.Properties.Settings.Default.Save();
            }
            catch { }
            
            base.OnClosing(e);
        }

        /// <summary>
        /// Opens a select folder dialog where you can choose the folder to save the images to.
        /// </summary>
        /// <param name="sender">The event sender object (here: select folder button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_SelectFolder_Click(object sender, EventArgs e)
        {
            try
            {
                folderBrowserDialog.SelectedPath = Path.GetDirectoryName(textBox_SaveFolder.Text);
            }
            catch { }

            if (folderBrowserDialog.ShowDialog(this) == DialogResult.OK)
            {
                textBox_SaveFolder.Text = folderBrowserDialog.SelectedPath;
                textBox_ExampleFileName.Text = textBox_filenamePrefix.Text + string.Format(numberFormat, numericUpDown_ImageStartNumber.Value) + ".bmp";
            }
        }

        /// <summary>
        /// Select the number the first image file name ends with. Next images will be numbered consecutively.
        /// </summary>
        /// <param name="sender">The event sender object (here: image start number spinbox).</param>
        /// <param name="e">The event argument (here: value changed).</param>
        private void numericUpDown_ImageStartNumber_ValueChanged(object sender, EventArgs e)
        {
            textBox_ExampleFileName.Text = textBox_filenamePrefix.Text + string.Format(numberFormat, numericUpDown_ImageStartNumber.Value) + ".bmp";
        }

        /// <summary>
        /// After clicking the button, images will be saved. File name starts with selected prefix and ends
        /// numbered starting with the selected number and numbered consecutively.
        /// </summary>
        /// <param name="sender">The event sender object (here: start grabbing button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_StartGrabbing_Click(object sender, EventArgs e)
        {
            //button_StartGrabbing.Enabled = false;
            button_StopGrabbing.Enabled = true;
            button_SelectFolder.Enabled = false;
            textBox_filenamePrefix.Enabled = false;
            numericUpDown_ImageStartNumber.Enabled = false;
            //button_TriggerSensor.Enabled = false;

            // offset counter is set to zero to start numbering from selected number on.
            _offsetCounter = 0;
            // save images flag is set to true to indicate save incoming images from now on.
            _saveImages = true;
        }

        /// <summary>
        /// Stop saving incoming images.
        /// </summary>
        /// <param name="sender">The event sender object (here: stop grabbing button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_StopGrabbing_Click(object sender, EventArgs e)
        {
            button_StartGrabbing.Enabled = true;
            button_StopGrabbing.Enabled = false;
            button_SelectFolder.Enabled = true;
            textBox_filenamePrefix.Enabled = true;
            numericUpDown_ImageStartNumber.Enabled = true;
            button_TriggerSensor.Enabled = true;
            
            // save images flag is set to false to indicate not to save incoming images from now on.
            _saveImages = false;
        }

        /// <summary>
        /// Send trigger command to sensor. See parameter list to get the parameter names to send such commands.
        /// </summary>
        /// <param name="sender">The event sender object (here: trigger sensor button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_TriggerSensor_Click(object sender, EventArgs e)
        {
            _sensor.SetSpecificSingleParameter(1, "Command", "TriggerStart", "1");
        }

        /// <summary>
        /// Event invoked when an image is received from sensor.
        /// </summary>
        /// <param name="imageType">The type of the image (current image, teach image, error image).</param>
        /// <param name="frameCounter">The frame number of the received image.</param>
        /// <param name="image">The received image.</param>
        void _sensor_ImageReceived(ImageTypes imageType, uint frameCounter, Bitmap image)
        {
            // Give handle back from connection thread to GUI thread to be able to access GUI elements.
            if (this.InvokeRequired)
            {
                this.Invoke(new ImageReceivedEventHandler(_sensor_ImageReceived), new object[] { imageType, frameCounter, image });
                return;
            }

            // show images
            pictureBox.Image = image;

            // only save images when start saving button was clicked -> flag is set.
            if (_saveImages)
            {
                // try saving images in selected folder with selected prefix and consecutive number.
                try
                {
                    image.Save(textBox_SaveFolder.Text + "\\" + textBox_filenamePrefix.Text + string.Format(numberFormat, numericUpDown_ImageStartNumber.Value + _offsetCounter) + ".bmp", ImageFormat.Bmp);
                    ++_offsetCounter;
                }
                catch (Exception e)
                {
                    MessageBox.Show(string.Format("Error saving image to file: {0}", e.Message));
                }
            }
        }

        private void button_GetLastImg_Click(object sender, EventArgs e)
        {
            _sensor.SetSpecificSingleParameter(1,"Command", "GetLastImg", "1");
        }

        private void button_ErrorImg_Click(object sender, EventArgs e)
        {
            _sensor.SetSpecificSingleParameter(1,"Command", "GetErrImg", "1");
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox showImg = sender as CheckBox;

            if (showImg.Checked)
            {
                _sensor.SetSpecificSingleParameter(1, "System", "ImageTransferActive", "1");
            }
            else
            {
                _sensor.SetSpecificSingleParameter(1, "System", "ImageTransferActive", "0");
            }
            
        }

		/// <summary>
        /// The number format used in the file names. The image counter is formatted according to this
		/// setting, which means as an integer number with 8 characters width and zero-padding.
        /// </summary>
        private const string numberFormat = "{0:00000000}";

		/// <summary>
        /// Event handler for the periodic trigger check-box. If checked, a trigger signal is
		/// periodically sent to the sensor. At the same time, the trigger button is disabled.
        /// </summary>
        /// <param name="sender">The object that emitted the event.</param>
        /// <param name="e">Additional event arguments.</param>
        private void checkBox_PeriodicTrigger_CheckedChanged(object sender, EventArgs e)
        {
            bool periodicTrigger = checkBox_PeriodicTrigger.CheckState.Equals(CheckState.Checked);
            if (periodicTrigger)
            {
                button_TriggerSensor.Enabled = false;
                timer1.Start();
            }
            else
            {
                timer1.Stop();
                button_TriggerSensor.Enabled = true;
            }
        }

		/// <summary>
        /// Event handler for the periodic trigger value field. The field stores the
		/// interval in ms at which a periodic trigger signal is sent to the sensor
		/// if the corresponding check-box is also set.
        /// </summary>
        /// <param name="sender">The object that emitted the event.</param>
        /// <param name="e">Additional event arguments.</param>
        private void numericUpDown_PeriodicTrigger_ValueChanged(object sender, EventArgs e)
        {
            timer1.Interval = (int)numericUpDown_PeriodicTrigger.Value;
        }

		/// <summary>
        /// The timer tick function for the periodic trigger.
		/// This simply sends a trigger command to the sensor each 
		/// interval in ms at which a periodic trigger signal is sent to the sensor
		/// if the corresponding check-box is also set.
        /// </summary>
        /// <param name="sender">The object that emitted the event.</param>
        /// <param name="e">Additional event arguments.</param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            _sensor.SetSpecificSingleParameter(1, "Command", "TriggerStart", "1");
        }
    }
}
