﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using PF.Foundation.VsxFactory;
using System.IO;

namespace LR300_Simple_dotNetInterface
{
    /*************************************************************************************************
     * 
     * Simple example to show the software interface. This programm visualizes the sensor images,
     * writes the line data in a csv- file and send specific parameter to the sensor.
     * The parameter file can be created by the Vision Configurator.
     * 
     * Author: Christoph Blankenburg
     * Date: 27.09.2011
     * 
     *
     **************************************************************************************************/
    public partial class MainForm : Form
    {
        //private class member thar represents the LineRunner300
        private PF.Foundation.VsxFactory.PFVsxFactoryVCCustom _LR300;
        //fixed portnumber 50005
        private const Int32 _portnumber = 50005;
        private FileStream _csvFileStream;
        private StreamWriter _csvStreamWriter;

        public MainForm()
        {
            //automatically created by the VS- GUI- Designer (see Form1.Designer.cs)
            InitializeComponent();

            //creates the LineRunner reference object
            _LR300 = new PFVsxFactoryVCCustom();
            //defines an eventhandler to handle the images from the sensor
            _LR300.ImageReceived += new ImageReceivedEventHandler(_LR300_ImageReceived);
            //defines an eventhandler to handle the line data from the sensor
            _LR300.LineDataReceived += new LineDataReceivedEventHandler(_LR300_LineDataReceived);

            //dialog to selct a csv- file, where the measurement data can be saved
            saveFileDialog1.Filter = "csv files (*.csv)|*.csv"; //Default csv- files
            saveFileDialog1.DefaultExt = "csv";

            saveFileDialog1.OverwritePrompt = false;
        }
        
        /// <summary>
        /// Is called when the connect button is pressed
        /// </summary>
        /// <param name="sender">Connect button</param>
        /// <param name="e">Connect button argument</param>
        private void buttonConnect_Click(object sender, EventArgs e)
        {
            //reads IP from textbox
            String ip = textBoxIP.Text;
            String headline = "";
            if (!_LR300.Connect(ip, _portnumber))
                LogMesages.Text += "Warning:\n No connection available!\n";
            else
            {
                LogMesages.Text += "Sensor connected!\n Press Trigger to get data.\n";
                if (!File.Exists(textPathCsv.Text))
                {
                    headline = "X ; Z ; Valid; Intensity \n";
                    LogMesages.Text += "\n New File " + textPathCsv.Text + " is created\n";
                }

                try
                {
                    //open csv- file in appending mode, so old data will not be deleted
                    if (_csvFileStream == null || !_csvFileStream.CanRead)
                    {
                        _csvFileStream = new FileStream(textPathCsv.Text, FileMode.Append);
                    }
                    if (_csvStreamWriter == null || _csvStreamWriter.BaseStream == null)
                    {
                        _csvStreamWriter = new StreamWriter(_csvFileStream);
                    }

                    // its faster with no auto flush.
                    _csvStreamWriter.AutoFlush = false;
                    _csvStreamWriter.Write(headline);
                }
                catch
                {
                    LogMesages.Text += "\n Cannot create or open  " + textPathCsv.Text + "\n";
                }

                //load Config- File
                if (File.Exists(textParameter.Text))
                {
                    _LR300.SetAllParameters(textParameter.Text);
                }
                else
                {
                    LogMesages.Text += "\n File " + textParameter.Text + " doesn't exist!\n";
                }
                buttonTrigger.Enabled = true;
                buttonConnect.Enabled = false;
            }
        }

        /// <summary>
        /// Is called, when the sensor is sending an image. This image should be shown in the image box.
        /// </summary>
        /// <param name="frameCounter">Index of the image</param>
        /// <param name="image">Image form the sensor</param>
        void _LR300_ImageReceived(ImageTypes type, uint frameCounter, Bitmap image)
        {
            //is needed to have access on the windows GUI elements (thread safty)
            if (this.InvokeRequired)
            {
                //it doesn't matter wich GUI element you are invoking;
                //the form of the statement is every time the same.
                this.LogMesages.Invoke(new ImageReceivedEventHandler(_LR300_ImageReceived), new object[] {type, frameCounter, image });
                return;
            }
            //the picture box get the image from the Sensor
            imageControl.Image = image;

        }
        /// <summary>
        /// Is called, when the sensor is sending line data. From this line data only the X and Z component will be written in the text box.
        /// </summary>
        /// <param name="version">Version of the sensor</param>
        /// <param name="status">Binary data, that shows, which in- and outputs are on (1) or off (0)</param>
        /// <param name="frameCounter">Number of the sent line data</param>
        /// <param name="results">Encapsulate the line data of different lines (if supported)</param>
        void _LR300_LineDataReceived(ushort version, ushort status, ushort frameCounter, PF.Types.LineMulti results)
        {

            //is needed to have access on the windows GUI elements (thread safty)
            if (this.InvokeRequired)
            {
                //it doesn't matter wich GUI element you are invoking
                this.LogMesages.Invoke(new LineDataReceivedEventHandler(_LR300_LineDataReceived), new object[] { version, status, frameCounter, results });
                return;
            }

            //extract the line data from the LineMulti class, normally it's the zero line.    
            List<PF.Types.Coordinate> points = results.Lines[0].Points;
            String lineString;
            //writes the Z and X Values of the array in the CSV file 
            if (_csvStreamWriter != null)
            {
                foreach (PF.Types.Coordinate p in points)
                {
                    lineString =
                    p.WorldCoordinate.X.ToString() + ";" +
                    p.WorldCoordinate.Z.ToString() + ";" +
                    p.Valid + ";" +
                    p.Intensity;
                    //writes string in CSV-File
                    _csvStreamWriter.WriteLine(lineString);
                }
                _csvStreamWriter.Flush();
            }
            else
            {
                LogMesages.Text += "\n Warning:\n CSV- file not found.\n";
            }
        }


        /// <summary>
        /// Before the form is closing this methode disconnect the Ethernet connection
        /// </summary>
        /// <param name="sender">FormClosing Button</param>
        /// <param name="e">The argument of the FormClosing button</param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            buttonDisconnect_Click(sender, e);
        }

        /// <summary>
        /// Is called, when the trigger button is pressed. Sends a trigger command to the sensor.
        /// </summary>
        /// <param name="sender">button object</param>
        /// <param name="e">Argument of the button object</param>
        private void buttonTrigger_Click(object sender, EventArgs e)
        {
            //trigger sensor
            _LR300.SetSpecificSingleParameter(1,"Command", "TriggerStart", "1");
        }

        /// <summary>
        /// Is called when the disconnect button is pressed
        /// </summary>
        /// <param name="sender">Disconnect button</param>
        /// <param name="e">Argument of the button object</param>
        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            //check if stream writer is open
            if (_csvStreamWriter != null)
            {
                _csvStreamWriter.Close();
            }
            //check if connected
            if (_LR300.Connected)
            {
                _LR300.Disconnect();
            }
            buttonConnect.Enabled = true;
            buttonTrigger.Enabled = false;
        }

        /// <summary>
        /// Select path to existing or new CSV- File.
        /// </summary>
        /// <param name="sender">button object</param>
        /// <param name="e">argument of button</param>
        private void buttonSelectPathCSV_Click(object sender, EventArgs e)
        {
            DialogResult result;

            //if connected, reconnect sensor
            if (_LR300.Connected)
            {
                result = MessageBox.Show("\n This operation disconnecting the sensor.\n Press ok to continue.", "Warning", MessageBoxButtons.OKCancel);
                if (result == DialogResult.Abort)
                    return;
                else
                    buttonDisconnect_Click(sender, e);
            }

            saveFileDialog1.FileName = textPathCsv.Text;
            saveFileDialog1.ShowDialog();
            if (saveFileDialog1.CheckFileExists)
            {
                LogMesages.Text += "\n Data will be append on the CSV- file.\n";
            }
            textPathCsv.Text = saveFileDialog1.FileName;
        }
        /// <summary>
        /// Slecting path to configfile, if file dosn't exists default values will be choosen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSelectPathConfig_Click(object sender, EventArgs e)
        {
            openFileDialog1.Filter = "LR300 config files (*.LR300_pfc)|*.LR300_pfc";
            openFileDialog1.FileName = textParameter.Text;
            openFileDialog1.ShowDialog();
            if (openFileDialog1.CheckFileExists)
            {
                textParameter.Text = openFileDialog1.FileName;
                _LR300.SetAllParameters(textParameter.Text);
            }
            else
            {
                LogMesages.Text += "\n Warning:\n config file not found.\n";
            }           
        }
    }
}
