﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using PF.Foundation.VsxFactory;

namespace LR300_Simple_dotNetInterface
{
    /*************************************************************************************************
     * 
     * Simple example to show the software interface. This program visualizes the sensor images and
     * writes the line data in a text box.
     * 
     * Author: Christoph Blankenburg
     * Date: 27.06.2011
     * 
     **************************************************************************************************/
    public partial class MainForm : Form
    {
       //private class member that represents the LineRunner300
        PF.Foundation.VsxFactory.PFVsxFactoryVCCustom _LR300;
        //fixed port number 50005 see constructor
        const Int32 _portnumber = 50005;
      
       
        public MainForm()
        {
            //automatically created by the VS- GUI- Designer (see MainForm.designer.cs)
            InitializeComponent();
            buttonTrigger.Enabled = false;

            //Creates the LineRunner reference class
            _LR300 = new PFVsxFactoryVCCustom();
            //defines an event handler to handle the images from the sensor
            _LR300.ImageReceived += new ImageReceivedEventHandler(_LR300_ImageReceived);
            //defines an event handler to handle the line data from the sensor
            _LR300.LineDataReceived += new LineDataReceivedEventHandler(_LR300_LineDataReceived);

            _LR300.LogDataReceived += new LogDataReceivedEventHandler(_LR300_LogDataReceived);

        }

        void _LR300_LogDataReceived(string logData, LogMessageTypes logType)
        {
                        //is needed to have access on the windows GUI elements (thread safety)
            if (this.InvokeRequired)
            {
                this.textBox1.Invoke(new LogDataReceivedEventHandler(_LR300_LogDataReceived), new object[] { logData, logType });
                return;
            }
            textBox1.AppendText(logData);
            textBox1.AppendText( "\r\n" );
            textBox1.ScrollToCaret();

            //throw new NotImplementedException();
        }

        /// <summary>
        /// Is called, when the sensor is sending an image. This image should be shown in the image box.
        /// </summary>
        /// <param name="frameCounter">Index of the image</param>
        /// <param name="image">Image form the sensor</param>
        void _LR300_ImageReceived(ImageTypes imgType, uint frameCounter, Bitmap image)
        {
            //is needed to have access on the windows GUI elements (thread safety)
            if (this.InvokeRequired)
            {
                //it doesn't matter wich GUI element you are invoking;
                //the form of the statement is every time the same.
                this.textBox1.Invoke(new ImageReceivedEventHandler(_LR300_ImageReceived), new object[] {imgType, frameCounter, image});
                return;
            }
            //the picture box get the image from the Sensor
            imageControl.Image = image;
          
        }
        /// <summary>
        /// Is called, when the sensor is sending line data. From this line data only the X and Z component will be written in the text box.
        /// </summary>
        /// <param name="version">Version of the sensor</param>
        /// <param name="status">Binary data, that shows, which in- and outputs are on (1) or off (0)</param>
        /// <param name="frameCounter">Number of the sent line data</param>
        /// <param name="results">Encapsulate the line data of different lines (if supported)</param>
        void _LR300_LineDataReceived(ushort version, ushort status, ushort frameCounter, PF.Types.LineMulti results)
        {
            
            //is needed to have access on the windows GUI elements (thread safty)
            if (this.InvokeRequired)
            {
                //it doesn't matter which GUI element you are invoking
                this.textBox2.Invoke(new LineDataReceivedEventHandler(_LR300_LineDataReceived), new object[]{version,status,frameCounter,results});
                return;
            }

            //Extract the line data from the LineMulti class    
            List<PF.Types.Coordinate> points = results.Lines[0].Points;

            // Output XYZ of 1st line point
            textBox2.AppendText(    "Z:" + points[0].WorldCoordinate.Z.ToString("F3") + " " +
                                        "X:" + points[0].WorldCoordinate.X.ToString("F3") + " " +
                                        "Y:" + points[0].WorldCoordinate.Y.ToString("F3") + "\r\n" );

            textBox2.ScrollToCaret();

/*            Writes the Z and X Values of the array in Windows text box 
              foreach (PF.Types.Coordinate p in points)
              {                    
                richTextBox1.Text += points[ii].WorldCoordinate.Z.ToString("F3")
                                        + "     "
                                        + points[ii].WorldCoordinate.X.ToString("F3")
                                        + "     "
                                        + points[ii].WorldCoordinate.Y.ToString("F3")
                                        +"\n";
              }
 */ 
         }

      
        /// <summary>
        /// Before the form is closing this method disconnect the Ethernet connection
        /// </summary>
        /// <param name="sender">FormClosing Button</param>
        /// <param name="e">The argument of the FormClosing button</param>
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            buttonDisconnect_Click(sender, e);
        }

        /// <summary>
        /// Is called, when the trigger button is pressed. Sends a trigger command to the sensor.
        /// </summary>
        /// <param name="sender">button object</param>
        /// <param name="e">Argument of the button object</param>
        private void buttonTrigger_Click(object sender, EventArgs e)
        {
            //see manual of LR300 for more information
            _LR300.SetSpecificSingleParameter(1,"Command", "TriggerStart", "1");
            
        }

        /// <summary>
        /// Is called when the disconnect button is pressed
        /// </summary>
        /// <param name="sender">Disconnect button</param>
        /// <param name="e">Argument of the button object</param>
        private void buttonDisconnect_Click(object sender, EventArgs e)
        {
            this.Text = "LR300";
            buttonConnect.Enabled = true;
            buttonDisconnect.Enabled = false;
            buttonTrigger.Enabled = false;
            textBoxIP.Enabled = true;
            //Check if connected
            if (_LR300.Connected)
                _LR300.Disconnect();
        }

        /// <summary>
        /// Is called when the connect button is pressed
        /// </summary>
        /// <param name="sender">connect button</param>
        /// <param name="e">connect button argument</param>
        private void buttonConnect_Click(object sender, EventArgs e)
        {
            //reads IP from text box
            String ip = textBoxIP.Text;
            if ( _LR300.Connect(ip, _portnumber))
            {
                this.Text = "LR300 - Connected";
                buttonConnect.Enabled = false;
                buttonDisconnect.Enabled = true;
                buttonTrigger.Enabled = true;
                textBoxIP.Enabled = false;
                //MessageBox.Show("Sensor connected!\n Press Trigger to get data.");
            }
            else
            {
                //MessageBox.Show("No connection available", "Waring");
            }
        }
     

    }
}
