﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using PF.Foundation.VsxFactory;
using PF.Foundation.Protocol.Xml;

namespace VsxFactoryTestProject
{
    /// <summary>
    /// Simple class to show the using of the VSX Software Interface. The program shows how to set parameters on sensor and
    /// how to visualize image and shape data.
    /// </summary>
    /// <remarks>
    /// @author     A. Brodmann
    /// @date       27.08.2013
    /// @copyright	2013 Pepperl + Fuchs GmbH, Mannheim<br/>
    ///             Proprietary material, all rights reserved, use or disclosure forbidden.
    /// </remarks>
    public partial class MainForm : Form
    {
        private PFVsxFactoryVCCustom _sensor;
        
        /// <summary>
        /// Constructor of the GUI.
        /// </summary>
        /// <remarks>
        /// Creates a new instance of the PFVsxFactoryVCCustom class and adds the event handlers to it.
        /// </remarks>
        public MainForm()
        {
            InitializeComponent();
            _sensor = new PFVsxFactoryVCCustom();
            // Add events. See event functions for detailed descriptions.
            _sensor.SpecificSingleDataReceived += new SpecificSingleDataReceivedEventHandler(_sensor_SpecificSingleDataReceived);
            _sensor.ParameterDataReceived += new ParameterDataReceivedEventHandler(_sensor_ParameterDataReceived);
            _sensor.ImageReceived += new ImageReceivedEventHandler(_sensor_ImageReceived);
            _sensor.SensorResultDataReceived += new SensorResultDataReceivedEventHandler(_sensor_SensorResultDataReceived);
            _sensor.ShapeDataReceived += new ShapeDataReceivedEventHandler(_sensor_ShapeDataReceived);            
        }

        /// <summary>
        /// Opens a TCP/IP connection with the sensor. If the connection attempt is successful, the whole parameter will be requested and stored internally.
        /// </summary>
        /// <param name="sender">The event sender object (here: Connect button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_Connect_Click(object sender, EventArgs e)
        {
            // Call the connect-function with a valid ip and port to start connection.
            if (_sensor.Connect(textBox_Ip.Text, 50005))
            {
                this.Text = "OPX - Connected";                
            }
            else
            {
                MessageBox.Show("Error connecting with sensor.");
                return;
            }
            button_connect.Enabled = false;
            button_disconnect.Enabled = true;
            button_TriggerSensor.Enabled = true;
            trackBar_FlashValue.Enabled = true;
        }

        /// <summary>
        /// Disconnects TCP/IP connection.
        /// </summary>
        /// <param name="sender">The event sender object (here: Disconnect button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_Disconnect_Click(object sender, EventArgs e)
        {
            if (_sensor.Connected)
            {
                _sensor.Disconnect();

                this.Text = "OPX";
                button_connect.Enabled = true;
                button_disconnect.Enabled = false;
                button_TriggerSensor.Enabled = false;
                trackBar_FlashValue.Enabled = false;
            }
        }

        /// <summary>
        /// Event of Trigger button in GUI.
        /// </summary>
        /// <param name="sender">The event sender object (here: Trigger button).</param>
        /// <param name="e">The event argument (here: click).</param>
        private void button_TriggerStart_Click(object sender, EventArgs e)
        {
            // Send single parameter, given by config ID, parameter ID and value to the sensor. Get the IDs from parameter list. 
            // Here, value is "1" because a button is clicked.
            _sensor.SetSpecificSingleParameter(1, "Command", "TriggerStart", "1");
        }

        /// <summary>
        /// Event of TrackBar in GUI.
        /// </summary>
        /// <param name="sender">The event sender object (here: TrackBar).</param>
        /// <param name="e">The event argument (here: slider change).</param>
        private void trackBar_FlashValue_Scroll(object sender, EventArgs e)
        {
            // update the label with the TrackBar value
            label_FlashValue.Text = Convert.ToString(trackBar_FlashValue.Value);
            // send new TrackBar value 
            _sensor.SetSpecificSingleParameter(1, "Camera", "FlashTime", Convert.ToString(trackBar_FlashValue.Value));
        }

        /// <summary>
        /// When closing GUI, disconnect sensor.
        /// </summary>
        /// <param name="e">The cancel event.</param>
        protected override void OnClosing(CancelEventArgs e)
        {
            if (_sensor.Connected)
                _sensor.Disconnect();
            base.OnClosing(e);
        }

        /// <summary>
        /// Event invoked after a single parameter call.
        /// </summary>
        /// <param name="version">The parameters version.</param>
        /// <param name="configId">The parameters config ID.</param>
        /// <param name="parameterId">The parameters parameter ID.</param>
        /// <param name="value">The paramaters value.</param>
        private void _sensor_SpecificSingleDataReceived(ushort version, string configId, string parameterId, string value)
        {
            // Give handle back from connection thread to GUI thread to be able to access GUI elements.
            if (this.InvokeRequired)
            {
                this.Invoke(new SpecificSingleDataReceivedEventHandler(_sensor_SpecificSingleDataReceived), new object[] { version, configId, parameterId, value });
                return;
            }

            // Check if desired parameter is received.
            // Show results in the corresponding labels.
            if ((configId == "Camera") && (parameterId == "FlashTime"))
            {
                // Set Label and TrackBar to the corresponding value
                label_FlashValue.Text = value;
                trackBar_FlashValue.Value = Convert.ToInt32(value);
            }
        }

        /// <summary>
        /// Event invoked when parameters are received from sensor. You can check all your parameters for changes. See documentation for a parameter list.
        /// </summary>
        /// <param name="modifier">Shows what function was called before to get the parameters.</param>
        void _sensor_ParameterDataReceived(DataModifier modifier)
        {
            _sensor.GetSpecificSingleParameter(1, "Camera", "FlashTime");
        }

        /// <summary>
        /// Event invoked when an image is received from sensor.
        /// </summary>
        /// <param name="imageType">The type of the image (current image, teach image, error image).</param>
        /// <param name="frameCounter">The frame number of the received image.</param>
        /// <param name="image">The received image.</param>
        void _sensor_ImageReceived(ImageTypes imageType, uint frameCounter, Bitmap image)
        {
            // Give handle back from connection thread to GUI thread to be able to access GUI elements.
            if (this.InvokeRequired)
            {
                this.Invoke(new ImageReceivedEventHandler(_sensor_ImageReceived), new object[] { imageType, frameCounter, image });
                return;
            }

            // Set the received image.
            pictureBox_Image.Image = image;

            // Look for shapes with the same frame number.
            IList<ElementShapeBase> shapeList = _sensor.GetShapes(frameCounter);

            // If shapes are found, draw them into the image.
            if (shapeList != null)
            {
                try
                {
                    Bitmap bitmap = new Bitmap(pictureBox_Image.Image);
                    using (Graphics graphics = Graphics.FromImage(bitmap))
                    {
                        // all shapes are from base type ElementShapeBase
                        foreach (ElementShapeBase shape in shapeList)
                        {
                            string configId = shape.ConfigurationId;
                            string parameterId = shape.Id;

                            // if shape is from type rectangle, draw a rectangle with given coordinates and size.
                            if (shape.Type == Vsx.ParameterTypes.Rectangle)
                            {
                                graphics.DrawRectangle(new Pen(shape.Color, 3), new Rectangle(new Point((int)shape.ShapeLocation.X, (int)shape.ShapeLocation.Y), 
                                    (shape as ElementShapeRectangle).Size));
                            }
                            // if shape is from type text, draw the given text on the given coordinates.
                            else if (shape.Type == Vsx.ParameterTypes.Text)
                            {
                                graphics.DrawString((shape as ElementShapeText).Text, new Font("Arial", (shape as ElementShapeText).FontSize), new SolidBrush(shape.Color), 
                                    new PointF(shape.ShapeLocation.X, shape.ShapeLocation.Y));
                            }
                        }
                        // Set the image with the shapes drawed in.
                        pictureBox_Image.Image = bitmap;
                    }
                }
                catch { }
            }
        }

        /// <summary>
        /// Event invoked when shape data is received from sensor.
        /// </summary>
        /// <param name="frameCounter">The frame number of the received shape data.</param>
        void _sensor_ShapeDataReceived(ushort frameCounter)
        {
            // Give handle back from connection thread to GUI thread to be able to access GUI elements.
            if (this.InvokeRequired)
            {
                this.Invoke(new ShapeDataReceivedEventHandler(_sensor_ShapeDataReceived), new object[] { frameCounter });
                return;
            }

            // Get the whole received shape list, identified by the frame counter.
            IList<ElementShapeBase> shapeList = _sensor.GetShapes(frameCounter);

            // Look for an image with the same frame number.
            Bitmap bitmap = _sensor.GetImage(frameCounter);

            // If there is one image, draw shapes in it.
            if (bitmap != null)
            {
                try
                {
                    using (Graphics graphics = Graphics.FromImage(bitmap))
                    {
                        // All shapes are from base type ElementShapeBase
                        foreach (ElementShapeBase shape in shapeList)
                        {
                            // If shape is ElementShapeRectangle, draw a rectangle with given coordinates and size.
                            if (shape.Type == Vsx.ParameterTypes.Rectangle)
                            {
                                graphics.DrawRectangle(new Pen(shape.Color, 3), new Rectangle(new Point((int)shape.ShapeLocation.X, (int)shape.ShapeLocation.Y),
                                    (shape as ElementShapeRectangle).Size));
                            }
                            // If shape is ElementShapeText, draw the given text on the given coordinates.
                            else if (shape.Type == Vsx.ParameterTypes.Text)
                            {
                                graphics.DrawString((shape as ElementShapeText).Text, new Font("Arial", (shape as ElementShapeText).FontSize), new SolidBrush(shape.Color), 
                                    new PointF(shape.ShapeLocation.X, shape.ShapeLocation.Y));
                            }
                        }
                        // Set the image with the shapes drawed in.
                        pictureBox_Image.Image = bitmap;
                    }
                }
                catch { }
            }
        }

        /// <summary>
        /// Event invoked when result data is received from sensor.
        /// </summary>
        /// <param name="frameCounter">The number of the corresponding image.</param>
        /// <remarks>
        /// Each result parameter is identified by it's config ID and parameter ID. See sensor's result parameter list for all available result parameters.
        /// </remarks>
        void _sensor_SensorResultDataReceived(ushort frameCounter)
        {
            // Give handle back from connection thread to GUI thread to be able to access GUI elements.
            if (this.InvokeRequired)
            {
                this.Invoke(new SensorResultDataReceivedEventHandler(_sensor_SensorResultDataReceived), new object[] { frameCounter });
                return;
            }

            // Get the result list identified by the frame counter.
            IList<ElementResult> resultList = _sensor.GetResultList(frameCounter);
            
            ResultWindow1.Text = "-";
            ResultWindow2.Text = "-";
            ResultWindow3.Text = "-";
            ResultWindow4.Text = "-";
            label_Win1_Time.Text = "-";
            label_Win2_Time.Text = "-";
            label_Win3_Time.Text = "-";
            label_Win4_Time.Text = "-";

            // Set Label Text with corresponding values 
            foreach (ElementResult result in resultList)
            {
                if (result.ParameterId == "ResW1")
                    ResultWindow1.Text = result.Value;
                if (result.ParameterId == "ResW2")
                    ResultWindow2.Text = result.Value;
                if (result.ParameterId == "ResW3")
                    ResultWindow3.Text = result.Value;
                if (result.ParameterId == "ResW4")
                    ResultWindow4.Text = result.Value;
                if (result.ParameterId == "ResW1Time")
                    label_Win1_Time.Text = result.Value;
                if (result.ParameterId == "ResW2Time")
                    label_Win2_Time.Text = result.Value;
                if (result.ParameterId == "ResW3Time")
                    label_Win3_Time.Text = result.Value;
                if (result.ParameterId == "ResW4Time")
                    label_Win4_Time.Text = result.Value;
            }
        }
    }
}
